from trac.core import Component, implements
from trac.perm import IPermissionPolicy, IPermissionRequestor

class CustomAttachmentPolicy(Component):
    """Adds permissions for exclusive rights to create or delete and replace
       attachments, regardless who added / changed it.

       Everybody who has permission `MILESTONE_ATTACHMENT_CREATE` can delete / replace attachments
       attached to MILESTONES, regardless who added / changed it.

       Once this plugin is enabled, you'll have to insert it at the appropriate
       place in your list of permission policies, e.g.
       {{{
       [trac]
       permission_policies = AttachmentDeletePolicy, DefaultPermissionPolicy, LegacyAttachmentPolicy
       }}}
    """
    implements(IPermissionPolicy, IPermissionRequestor)
    
    # IPermissionPolicy methods
    def check_permission(self, action, username, resource, perm):
        ''' Special check_permission for CustomAttachmentPolicy '''
        # We add the special permissions pre-requisite for any action
        # other than special permissions themselves, as this would lead
        # to recursion.
        # (copied from sample-plugins/permissions/vulnerability_tickets.py)
        if action in ['TICKET_ATTACHMENT_CREATE',
                     'MILESTONE_ATTACHMENT_CREATE', 
                     'WIKI_ATTACHMENT_CREATE', 
                     'TICKET_ATTACHMENT_DELETE',
                     'MILESTONE_ATTACHMENT_DELETE', 
                     'WIKI_ATTACHMENT_DELETE',
                     'ATTACHMENT_ADMIN']: 
            return
        
        # self.log.info( "action: %s, resource: %s" % (action, resource) )
        # Check whether we're dealing with a ticket resource

        if action and action in ['ATTACHMENT_CREATE', 'ATTACHMENT_DELETE'] \
           and resource and resource.realm == 'attachment':
            parent = resource.parent.realm.upper()
            permission = '%s_%s' % (parent, action)
            if action == 'ATTACHMENT_CREATE':
                action_text = 'creating'
            else:
                action_text = 'deleting'
            if permission in perm or 'ATTACHMENT_ADMIN' in perm:
                self.log.info("granted perm for user %s %s attachment %s" % \
                                           (username, action_text, resource))
                return True
    
    # IPermissionRequestor methods    
    def get_permission_actions(self):
        ''' Returns new permissions '''
        return ['TICKET_ATTACHMENT_CREATE',
                'MILESTONE_ATTACHMENT_CREATE',
                'WIKI_ATTACHMENT_CREATE',
                'TICKET_ATTACHMENT_DELETE',
                'MILESTONE_ATTACHMENT_DELETE',
                'WIKI_ATTACHMENT_DELETE',
                'ATTACHMENT_ADMIN']
